from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger

from database import app
from covid19.blueprints.application.application_services import who_service, ecdc_service, rki_vaccination_service
from covid19.blueprints.application.application_services import rki_service_bundeslaender, rki_service_landkreise
from covid19.blueprints.application.application_services import admin_service, owid_service
from covid19.blueprints.application.application_workers import celery
from covid19.blueprints.application.application_model_transient import ApplicationPage

drop_and_create_data_again = True

app_admin = Blueprint('app_admin', __name__, template_folder='templates', url_prefix='/admin')


# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_admin.route('/')
def url_admin_index():
    page_info = ApplicationPage('Admin', "Covid19 Admin")
    return render_template(
        'admin/index.html',
        page_info=page_info)


@app_admin.route('/tasks')
def url_admin_tasks():
    page_info = ApplicationPage('Admin', "Tasks")
    return render_template(
        'admin/admin_tasks.html',
        page_info=page_info)


@app_admin.route('/info')
def url_admin_info():
    page_info = ApplicationPage('Admin', "Info")
    return render_template(
        'admin/admin_info.html',
        page_info=page_info)


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_admin_alive_message(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_alive_message [received] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_alive_message)"
    return result


@celery.task(bind=True)
def task_admin_database_drop_create(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_database_drop_create [start] ")
    logger.info("------------------------------------------------------------")
    who_service.task_database_drop_create()
    ecdc_service.task_database_drop_create()
    owid_service.task_database_drop_create()
    rki_vaccination_service.task_database_drop_create()
    admin_service.task_database_drop_create()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_database_drop_create [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_database_drop_create)"
    return result


@celery.task(bind=True)
def task_admin_import_all_files(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_import_all_files [start] ")
    logger.info("------------------------------------------------------------")
    who_service.task_import_all_files()
    ecdc_service.task_import_all_files()
    owid_service.task_import_all_files()
    rki_vaccination_service.task_import_all_files()
    # rki_service_bundeslaender.task_import_all_files()
    # rki_service_landkreise.task_import_all_files()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_import_all_files [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_import_all_files)"
    return result


@celery.task(bind=True)
def task_admin_update_dimension_tables_only(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_dimension_tables_only [start] ")
    logger.info("------------------------------------------------------------")
    who_service.update_dimension_tables_only()
    ecdc_service.update_dimension_tables_only()
    owid_service.update_dimension_tables_only()
    rki_vaccination_service.update_dimension_tables_only()
    # rki_service_bundeslaender.update_dimension_tables_only()
    # rki_service_landkreise.update_dimension_tables_only()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_dimension_tables_only [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_update_dimension_tables_only)"
    return result


@celery.task(bind=True)
def task_admin_update_fact_table_initial_only(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_fact_table_initial_only [start] ")
    logger.info("------------------------------------------------------------")
    who_service.full_update_fact_table()
    ecdc_service.full_update_fact_table()
    owid_service.full_update_fact_table()
    rki_vaccination_service.full_update_fact_table()
    # rki_service_bundeslaender.update_fact_table_initial_only()
    # rki_service_landkreise.update_fact_table_initial_only()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_fact_table_initial_only [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_update_fact_table_initial_only)"
    return result


@celery.task(bind=True)
def task_admin_update_fact_table_incremental_only(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_fact_table_incremental_only [start] ")
    logger.info("------------------------------------------------------------")
    who_service.service_update.update_fact_table()
    ecdc_service.service_update.update_fact_table()
    owid_service.service_update.update_fact_table()
    rki_vaccination_service.service_update.update_fact_table()
    # rki_service_bundeslaender.service_update.update_fact_table()
    # rki_service_landkreise.service_update.update_fact_table()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_fact_table_incremental_only [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_update_fact_table_incremental_only)"
    return result


@celery.task(bind=True)
def task_admin_update_star_schema_initial(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_star_schema_initial [start] ")
    logger.info("------------------------------------------------------------")
    who_service.run_update_star_schema_initial()
    ecdc_service.run_update_star_schema_initial()
    owid_service.run_update_star_schema_initial()
    rki_vaccination_service.run_update_star_schema_initial()
    # rki_service_bundeslaender.run_update_star_schema_initial()
    # rki_service_landkreise.run_update_star_schema_initial()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_star_schema_initial [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_update_star_schema_initial)"
    return result


@celery.task(bind=True)
def task_admin_update_star_schema_incremental(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_star_schema_incremental [start] ")
    logger.info("------------------------------------------------------------")
    who_service.run_update_star_schema_incremental()
    ecdc_service.run_update_star_schema_incremental()
    owid_service.run_update_star_schema_incremental()
    rki_vaccination_service.run_update_star_schema_incremental()
    rki_service_bundeslaender.run_update_star_schema_incremental()
    rki_service_landkreise.run_update_star_schema_incremental()
    logger.info("------------------------------------------------------------")
    logger.info(" task_admin_update_star_schema_incremental [done] ")
    logger.info("------------------------------------------------------------")
    self.update_state(state=states.SUCCESS)
    result = "OK (task_admin_update_star_schema_incremental)"
    return result

# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_admin.route('/task/alive_message')
def url_task_admin_alive_message():
    app.logger.info("url_task_admin_message_start [start]")
    task_admin_alive_message.apply_async()
    flash("alive_message_task started")
    app.logger.info("url_task_admin_message_start [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/database/dump')
def url_task_admin_database_dump():
    app.logger.info("url_task_admin_database_dump [start]")
    admin_service.run_admin_database_dump()
    flash("admin_service.run_admin_database_dump started")
    app.logger.info("url_task_admin_database_dump [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/database/reimport')
def url_task_admin_database_dump_reimport():
    app.logger.info("url_task_admin_database_dump_reimport [start]")
    admin_service.run_admin_database_dump_reimport()
    flash("admin_service.run_admin_database_import started")
    app.logger.info("url_task_admin_database_dump_reimport [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/database/drop_create')
def url_task_admin_database_dropcreate():
    app.logger.info("url_task_admin_database_dropcreate [start]")
    admin_service.run_admin_database_drop_and_create()
    flash("admin_service.run_admin_database_drop started")
    app.logger.info("url_task_admin_database_dropcreate [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/database/drop')
def url_task_admin_database_drop():
    flash("url_task_admin_database_drop [start]")
    app.logger.info("admin_service.run_admin_database_drop_and_create [start]")
    admin_service.run_admin_database_drop_and_create()
    app.logger.info("admin_service.run_admin_database_drop_and_create [done]")
    if drop_and_create_data_again:
        who_service.pretask_database_drop_create()
        ecdc_service.pretask_database_drop_create()
        owid_service.pretask_database_drop_create()
        rki_vaccination_service.pretask_database_drop_create()
        rki_service_bundeslaender.pretask_database_drop_create()
        flash("task_admin_database_drop_create async started")
        task_admin_database_drop_create.apply_async()
    app.logger.info("url_task_admin_database_drop [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/files/all/download')
def url_task_admin_download_all_files():
    app.logger.info("url_task_admin_download_all_files [start]")
    who_service.download_all_files()
    ecdc_service.download_all_files()
    owid_service.download_all_files()
    rki_vaccination_service.download_all_files()
    rki_service_bundeslaender.download_all_files()
    rki_service_landkreise.download_all_files()
    app.logger.info("url_task_admin_download_all_files [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/files/all/import')
def url_task_admin_import_all_files():
    app.logger.info("url_task_admin_import_all_files [start]")
    task_admin_import_all_files.apply_async()
    flash("task_admin_import_all_files async started")
    app.logger.info("task_admin_import_all_files async started")
    app.logger.info("url_task_admin_import_all_files [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/all/dimension_tables')
def url_task_admin_update_dimension_tables():
    app.logger.info("url_task_admin_update_dimension_tables [start]")
    task_admin_update_dimension_tables_only.apply_async()
    flash("task_admin_import_all_files async started")
    app.logger.info("task_admin_update_all_blueprints async started")
    app.logger.info("url_task_admin_update_dimension_tables [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/full/all/dimension_tables')
def url_task_admin_update_full_dimension_tables():
    app.logger.info("url_task_admin_update_full_dimension_tables [start]")
    task_admin_update_dimension_tables_only.apply_async()
    flash("task_admin_import_all_files async started")
    app.logger.info("task_admin_update_all_blueprints async started")
    app.logger.info("url_task_admin_update_full_dimension_tables [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/full/all/fact_tables')
def url_task_admin_update_full_fact_tables():
    app.logger.info("url_task_admin_update_full_fact_tables [start]")
    task_admin_update_fact_table_initial_only.apply_async()
    flash("task_admin_update_fact_table_initial_only async started")
    app.logger.info("task_admin_update_fact_table_initial_only async started")
    app.logger.info("url_task_admin_update_full_fact_tables [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/all/fact_tables')
def url_task_admin_update_fact_tables():
    app.logger.info("url_admin_update_fact_table_incremental_only [start]")
    task_admin_update_fact_table_incremental_only.apply_async()
    flash("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("url_admin_update_fact_table_incremental_only [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/full/star_schema')
def url_task_admin_full_update_star_schema():
    app.logger.info("url_task_admin_full_update_star_schema [start]")
    task_admin_update_star_schema_initial.apply_async()
    flash("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("url_task_admin_full_update_star_schema [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/star_schema')
def url_task_admin_update_star_schema():
    app.logger.info("url_task_admin_update_star_schema [start]")
    task_admin_update_star_schema_incremental.apply_async()
    flash("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("task_admin_update_fact_table_incremental_only async started")
    app.logger.info("url_task_admin_update_star_schema [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))


@app_admin.route('/task/update/do/all/initial')
def url_task_admin_full_update_do_all():
    app.logger.info("url_admin_update_do_all_initial [start]")
    who_service.download_files()
    flash("who_service.download_files() [done]")
    ecdc_service.download_all_files()
    flash("ecdc_service.download_all_files() [done]")
    owid_service.download_all_files()
    flash("owid_service.download_all_files() [done]")
    rki_vaccination_service.download_all_files()
    flash("rki_vaccination_service.download_all_files() [done]")
    rki_service_bundeslaender.download_all_files()
    flash("rki_service_bundeslaender.download_all_files() [done]")
    rki_service_landkreise.download_all_files()
    flash("rki_service_landkreise.download_all_files() [done]")
    task_admin_update_star_schema_initial.apply_async()
    flash(message="async task_admin_update_star_schema_initial [start]", category="warning")
    app.logger.warn("async task_admin_update_star_schema_initial [start]")
    app.logger.info("url_admin_update_do_all_initial [done]")
    return redirect(url_for('app_admin.url_admin_tasks'))
